@@ -196,7 +196,7 @@ impl Default for Options {
 }
 
 // A subset of options that are meant to be changed per pipeline.
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Default, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serialize", derive(serde::Serialize))]
 #[cfg_attr(feature = "deserialize", derive(serde::Deserialize))]
 pub struct PipelineOptions {
@@ -205,14 +205,6 @@ pub struct PipelineOptions {
     pub allow_point_size: bool,
 }
 
-impl Default for PipelineOptions {
-    fn default() -> Self {
-        PipelineOptions {
-            allow_point_size: true,
-        }
-    }
-}
-
 impl Options {
     fn resolve_local_binding(
         &self,
