@@ -739,7 +739,7 @@ impl<'w> BlockContext<'w> {
 
         // Perform the access, according to the bounds check policy.
         let access_id = match self.writer.bounds_check_policies.image {
-            crate::back::BoundsCheckPolicy::Restrict => {
+            crate::proc::BoundsCheckPolicy::Restrict => {
                 let (coords, level_id, sample_id) = self.write_restricted_coordinates(
                     image_id,
                     coordinates,
@@ -749,7 +749,7 @@ impl<'w> BlockContext<'w> {
                 )?;
                 access.generate(&mut self.writer.id_gen, coords, level_id, sample_id, block)
             }
-            crate::back::BoundsCheckPolicy::ReadZeroSkipWrite => self
+            crate::proc::BoundsCheckPolicy::ReadZeroSkipWrite => self
                 .write_conditional_image_access(
                     image_id,
                     coordinates,
@@ -758,7 +758,7 @@ impl<'w> BlockContext<'w> {
                     block,
                     &access,
                 )?,
-            crate::back::BoundsCheckPolicy::Unchecked => access.generate(
+            crate::proc::BoundsCheckPolicy::Unchecked => access.generate(
                 &mut self.writer.id_gen,
                 coordinates.value_id,
                 level_id,
@@ -799,6 +799,7 @@ impl<'w> BlockContext<'w> {
         result_type_id: Word,
         image: Handle<crate::Expression>,
         sampler: Handle<crate::Expression>,
+        gather: Option<crate::SwizzleComponent>,
         coordinate: Handle<crate::Expression>,
         array_index: Option<Handle<crate::Expression>>,
         offset: Option<Handle<crate::Constant>>,
@@ -816,7 +817,7 @@ impl<'w> BlockContext<'w> {
             crate::TypeInner::Image {
                 class: crate::ImageClass::Depth { .. },
                 ..
-            } => depth_ref.is_none(),
+            } => depth_ref.is_none() && gather.is_none(),
             _ => false,
         };
         let sample_result_type_id = if needs_sub_access {
@@ -853,8 +854,23 @@ impl<'w> BlockContext<'w> {
         let mut mask = spirv::ImageOperands::empty();
         mask.set(spirv::ImageOperands::CONST_OFFSET, offset.is_some());
 
-        let mut main_instruction = match level {
-            crate::SampleLevel::Zero => {
+        let mut main_instruction = match (level, gather) {
+            (_, Some(component)) => {
+                let component_id = self.get_index_constant(component as u32);
+                let mut inst = Instruction::image_gather(
+                    sample_result_type_id,
+                    id,
+                    sampled_image_id,
+                    coordinates_id,
+                    component_id,
+                    depth_id,
+                );
+                if !mask.is_empty() {
+                    inst.add_operand(mask.bits());
+                }
+                inst
+            }
+            (crate::SampleLevel::Zero, None) => {
                 let mut inst = Instruction::image_sample(
                     sample_result_type_id,
                     id,
@@ -874,7 +890,7 @@ impl<'w> BlockContext<'w> {
 
                 inst
             }
-            crate::SampleLevel::Auto => {
+            (crate::SampleLevel::Auto, None) => {
                 let mut inst = Instruction::image_sample(
                     sample_result_type_id,
                     id,
@@ -888,7 +904,7 @@ impl<'w> BlockContext<'w> {
                 }
                 inst
             }
-            crate::SampleLevel::Exact(lod_handle) => {
+            (crate::SampleLevel::Exact(lod_handle), None) => {
                 let mut inst = Instruction::image_sample(
                     sample_result_type_id,
                     id,
@@ -905,7 +921,7 @@ impl<'w> BlockContext<'w> {
 
                 inst
             }
-            crate::SampleLevel::Bias(bias_handle) => {
+            (crate::SampleLevel::Bias(bias_handle), None) => {
                 let mut inst = Instruction::image_sample(
                     sample_result_type_id,
                     id,
@@ -922,7 +938,7 @@ impl<'w> BlockContext<'w> {
 
                 inst
             }
-            crate::SampleLevel::Gradient { x, y } => {
+            (crate::SampleLevel::Gradient { x, y }, None) => {
                 let mut inst = Instruction::image_sample(
                     sample_result_type_id,
                     id,
@@ -1131,12 +1147,12 @@ impl<'w> BlockContext<'w> {
         let write = Store { image_id, value_id };
 
         match self.writer.bounds_check_policies.image {
-            crate::back::BoundsCheckPolicy::Restrict => {
+            crate::proc::BoundsCheckPolicy::Restrict => {
                 let (coords, _, _) =
                     self.write_restricted_coordinates(image_id, coordinates, None, None, block)?;
                 write.generate(&mut self.writer.id_gen, coords, None, None, block);
             }
-            crate::back::BoundsCheckPolicy::ReadZeroSkipWrite => {
+            crate::proc::BoundsCheckPolicy::ReadZeroSkipWrite => {
                 self.write_conditional_image_access(
                     image_id,
                     coordinates,
@@ -1146,7 +1162,7 @@ impl<'w> BlockContext<'w> {
                     &write,
                 )?;
             }
-            crate::back::BoundsCheckPolicy::Unchecked => {
+            crate::proc::BoundsCheckPolicy::Unchecked => {
                 write.generate(
                     &mut self.writer.id_gen,
                     coordinates.value_id,
